@@ -1,8 +1,9 @@
 package Zonemaster::Engine;
 
-use version; our $VERSION = version->declare("v4.7.1");
+use v5.16.0;
+use warnings;
 
-use 5.014002;
+use version; our $VERSION = version->declare("v6.0.0");
 
 BEGIN {
     # Locale::TextDomain (<= 1.20) doesn't know about File::ShareDir so give a helping hand.
@@ -16,6 +17,7 @@ BEGIN {
 
 use Class::Accessor "antlers";
 use Carp;
+
 use Zonemaster::Engine::Nameserver;
 use Zonemaster::Engine::Logger;
 use Zonemaster::Engine::Profile;
@@ -71,16 +73,16 @@ sub test_module {
 }
 
 sub test_method {
-    my ( $class, $module, $method, @arguments ) = @_;
+    my ( $class, $module, $method, $zname ) = @_;
 
-    return Zonemaster::Engine::Test->run_one( $module, $method, @arguments );
+    return Zonemaster::Engine::Test->run_one( $module, $method, $class->zone( $zname ) );
 }
 
 sub all_tags {
     my ( $class ) = @_;
     my @res;
 
-    foreach my $module ( 'Basic', sort { $a cmp $b } Zonemaster::Engine::Test->modules ) {
+    foreach my $module ( sort { $a cmp $b } Zonemaster::Engine::Test->modules ) {
         my $full = "Zonemaster::Engine::Test::$module";
         my $ref  = $full->metadata;
         foreach my $list ( values %{$ref} ) {
@@ -95,7 +97,7 @@ sub all_methods {
     my ( $class ) = @_;
     my %res;
 
-    foreach my $module ( 'Basic', Zonemaster::Engine::Test->modules ) {
+    foreach my $module ( Zonemaster::Engine::Test->modules ) {
         my $full = "Zonemaster::Engine::Test::$module";
         my $ref  = $full->metadata;
         foreach my $method ( sort { $a cmp $b } keys %{$ref} ) {
@@ -260,11 +262,14 @@ Runs all available tests and returns a list of L<Zonemaster::Engine::Logger::Ent
 
 Runs all available tests for the zone with the given name in the specified module.
 
-=item test_method($module, $method, @arguments)
+=item test_method($module, $method, $name)
 
-Run one particular test method in one particular module. The requested module must be in the list of active loaded modules (that is, not the Basic
-module and not a module disabled by the current policy), and the method must be listed in the metadata the module exports. If those requirements
-are fulfilled, the method will be called with the provided arguments.
+Run one particular test method in one particular module for one particular zone.
+The requested module must be in the list of currently enabled modules (that is,
+not a module disabled by the current profile), and the method must be listed in
+the metadata of the module exports.
+If those requirements are fulfilled, the method will be called with the provided
+arguments.
 
 =item zone($name)
 
