@@ -1,33 +1,59 @@
 package Zonemaster::Engine::Profile;
 
-use 5.014002;
-
-use strict;
+use v5.16.0;
 use warnings;
 
-use version; our $VERSION = version->declare( "v1.2.17" );
+use version; our $VERSION = version->declare( "v1.2.22" );
 
 use File::ShareDir qw[dist_file];
 use JSON::PP qw( encode_json decode_json );
-use Scalar::Util qw(reftype);
+use Scalar::Util qw(reftype looks_like_number);
 use File::Slurp;
 use Clone qw(clone);
 use Data::Dumper;
+use Net::IP::XS;
+use Log::Any qw( $log );
+use YAML::XS qw();
 
-use Zonemaster::Engine;
-use Zonemaster::Engine::Net::IP;
-use Zonemaster::Engine::Constants qw( $RESOLVER_SOURCE_OS_DEFAULT );
+$YAML::XS::Boolean = "JSON::PP";
+
+use Zonemaster::Engine::Constants qw( $DURATION_5_MINUTES_IN_SECONDS $DURATION_1_HOUR_IN_SECONDS $DURATION_4_HOURS_IN_SECONDS $DURATION_12_HOURS_IN_SECONDS $DURATION_1_DAY_IN_SECONDS $DURATION_1_WEEK_IN_SECONDS $DURATION_180_DAYS_IN_SECONDS );
+use Zonemaster::Engine::Validation qw( validate_ipv4 validate_ipv6 );
 
 my %profile_properties_details = (
+    q{cache} => {
+        type    => q{HashRef},
+        test    => sub {
+            my @allowed_keys = ( 'redis' );
+            foreach my $cache_database ( keys %{$_[0]} ) {
+                if ( not grep( /^$cache_database$/, @allowed_keys ) ) {
+                    die "Property cache keys have " . scalar @allowed_keys . " possible values: " . join(", ", @allowed_keys);
+                }
+
+                if ( not scalar keys %{ $_[0]->{$cache_database} } ) {
+                    die "Property cache.$cache_database has no items";
+                }
+                else {
+                    my @allowed_subkeys;
+                    if ( $cache_database eq 'redis' ) {
+                        @allowed_subkeys = ( 'server', 'expire' );
+                    }
+
+                    foreach my $key ( keys %{ $_[0]->{$cache_database} } ) {
+                        if ( not grep( /^$key$/, @allowed_subkeys ) ) {
+                            die "Property cache.$cache_database subkeys have " . scalar @allowed_subkeys . " possible values: " . join(", ", @allowed_subkeys);
+                        }
+
+                        die "Property cache.$cache_database.$key has a NULL or empty item" if not $_[0]->{$cache_database}->{$key};
+                        die "Property cache.$cache_database.$key has a negative value" if ( looks_like_number( $_[0]->{$cache_database}->{$key} ) and $_[0]->{$cache_database}->{$key} < 0 ) ;
+                    }
+                }
+            }
+        }
+    },
     q{resolver.defaults.debug} => {
         type    => q{Bool}
     },
-    q{resolver.defaults.dnssec} => {
-        type    => q{Bool}
-    },
-    q{resolver.defaults.edns_size} => {
-        type    => q{Num}
-    },
     q{resolver.defaults.igntc} => {
         type    => q{Bool}
     },
@@ -50,16 +76,26 @@ my %profile_properties_details = (
     q{resolver.defaults.usevc} => {
         type    => q{Bool}
     },
-    q{resolver.source} => {
+    q{resolver.defaults.timeout} => {
+        type    => q{Num}
+    },
+    q{resolver.source4} => {
         type    => q{Str},
         test    => sub {
-            if ( $_[0] ne $RESOLVER_SOURCE_OS_DEFAULT ) {
-                eval { Zonemaster::Engine::Net::IP->new( $_[0] ) };
-                if ( $@ ) {
-                    die "Property resolver.source must be an IP address or the exact string $RESOLVER_SOURCE_OS_DEFAULT";
-                }
+            unless ( $_[0] eq '' or validate_ipv4( $_[0] ) ) {
+                die "Property resolver.source4 must be a valid IPv4 address";
             }
-        }
+        },
+        default => q{}
+    },
+    q{resolver.source6} => {
+        type    => q{Str},
+        test    => sub {
+            unless ( $_[0] eq '' or validate_ipv6( $_[0] ) ) {
+                die "Property resolver.source6 must be a valid IPv6 address";
+            }
+        },
+        default => q{}
     },
     q{net.ipv4} => {
         type    => q{Bool}
@@ -127,6 +163,46 @@ my %profile_properties_details = (
     },
     q{test_cases} => {
         type    => q{ArrayRef}
+    },
+    q{test_cases_vars.dnssec04.REMAINING_SHORT} => {
+        type    => q{Num},
+        min     => 1,
+        default => $DURATION_12_HOURS_IN_SECONDS
+    },
+    q{test_cases_vars.dnssec04.REMAINING_LONG} => {
+        type    => q{Num},
+        min     => 1,
+        default => $DURATION_180_DAYS_IN_SECONDS
+    },
+    q{test_cases_vars.dnssec04.DURATION_LONG} => {
+        type    => q{Num},
+        min     => 1,
+        default => $DURATION_180_DAYS_IN_SECONDS
+    },
+    q{test_cases_vars.zone02.SOA_REFRESH_MINIMUM_VALUE} => {
+        type    => q{Num},
+        min     => 1,
+        default => $DURATION_4_HOURS_IN_SECONDS
+    },
+    q{test_cases_vars.zone04.SOA_RETRY_MINIMUM_VALUE} => {
+        type    => q{Num},
+        min     => 1,
+        default => $DURATION_1_HOUR_IN_SECONDS
+    },
+    q{test_cases_vars.zone05.SOA_EXPIRE_MINIMUM_VALUE} => {
+        type    => q{Num},
+        min     => 1,
+        default => $DURATION_1_WEEK_IN_SECONDS
+    },
+    q{test_cases_vars.zone06.SOA_DEFAULT_TTL_MAXIMUM_VALUE} => {
+        type    => q{Num},
+        min     => 1,
+        default => $DURATION_1_DAY_IN_SECONDS
+    },
+    q{test_cases_vars.zone06.SOA_DEFAULT_TTL_MINIMUM_VALUE} => {
+        type    => q{Num},
+        min     => 1,
+        default => $DURATION_5_MINUTES_IN_SECONDS
     }
 );
 
@@ -186,7 +262,7 @@ sub _set_value_to_nested_hash {
     my ( $hash_ref, $value, @path ) = @_;
 
     my $key = shift @path;
-    
+
     if (  ! exists $hash_ref->{$key} ) {
         $hash_ref->{$key} = {};
     }
@@ -218,6 +294,7 @@ sub default {
             $new->set( $property_name, $profile_properties_details{$property_name}{default} );
         }
     }
+
     return $new;
 }
 
@@ -320,6 +397,7 @@ sub merge {
             $self->_set( q{JSON}, $property_name, _get_value_from_nested_hash( $other_profile->{q{profile}}, split /[.]/, $property_name ) );
         }
     }
+
     return $other_profile->{q{profile}};
 }
 
@@ -344,6 +422,18 @@ sub to_json {
     return encode_json( $self->{q{profile}} );
 }
 
+sub from_yaml {
+    my ( $class, $yaml ) = @_;
+    my $data = YAML::XS::Load( $yaml );
+    return $class->from_json( encode_json( $data ) );
+}
+
+sub to_yaml {
+    my ( $self ) = @_;
+
+    return YAML::XS::Dump( $self->{q{profile}} );
+}
+
 sub effective {
     return $effective;
 }
@@ -377,7 +467,7 @@ section.
 Here is an example for updating the effective profile with values from
 a given file and setting all properties not mentioned in the file to
 default values.
-For details on the file format see the L</JSON REPRESENTATION> section.
+For details on the file format see the L</REPRESENTATIONS> section.
 
     use Zonemaster::Engine::Profile;
 
@@ -419,10 +509,7 @@ Update it to change the configuration.
 The effective profile is initialized with the default values declared
 in the L</PROFILE PROPERTIES> section.
 
-For the effective profile, all properties are always set (to valid
-values).
-This is based on the assumption that F<default.profile> specifies a
-valid value for each and every property.
+All properties in the effective profile are always set (to valid values).
 
 =head1 CLASS METHODS
 
@@ -453,6 +540,23 @@ Dies if the given string is illegal according to the L</JSON REPRESENTATION>
 section or if the property values are illegal according to the L</PROFILE
 PROPERTIES> section.
 
+=head2 from_yaml
+
+A constructor that returns a new profile with values parsed from a YAML string.
+
+    my $profile = Zonemaster::Engine::Profile->from_yaml( <<EOF
+    no_network: true
+    EOF
+    );
+
+The returned profile has set values for all properties specified in the
+given string.
+The remaining properties are unset.
+
+Dies if the given string is illegal according to the L</YAML REPRESENTATION>
+section or if the property values are illegal according to the L</PROFILE
+PROPERTIES> section.
+
 =head1 INSTANCE METHODS
 
 =head2 get
@@ -501,6 +605,14 @@ Serialize the profile to the L</JSON REPRESENTATION> format.
 
 Returns a string.
 
+=head2 to_yaml
+
+Serialize the profile to the L</JSON REPRESENTATION> format.
+
+    my $string = $profile->to_yaml();
+
+Returns a string.
+
 =head1 SUBROUTINES
 
 =head2 _get_profile_paths
@@ -538,9 +650,6 @@ If it is set it has a value that is valid for that specific property.
 Here is a listing of all the properties and their respective sets of
 valid values.
 
-Default values are listed here as specified in the distributed default
-profile JSON file.
-
 =head2 resolver.defaults.usevc
 
 A boolean. If true, only use TCP. Default false.
@@ -550,10 +659,6 @@ A boolean. If true, only use TCP. Default false.
 An integer between 1 and 255 inclusive. The number of seconds between retries.
 Default 3.
 
-=head2 resolver.defaults.dnssec
-
-A boolean. If true, sets the DO flag in queries. Default false.
-
 =head2 resolver.defaults.recurse
 
 A boolean. If true, sets the RD flag in queries. Default false.
@@ -573,7 +678,7 @@ flag set will be automatically resent over TCP. Default false.
 =head2 resolver.defaults.fallback
 
 A boolean. If true, UDP queries that get responses with the C<TC>
-flag set will be automatically resent over TCP or using EDNS. Default 
+flag set will be automatically resent over TCP or using EDNS. Default
 true.
 
 In ldns-1.7.0 (NLnet Labs), in case of truncated answer when UDP is used,
@@ -581,12 +686,23 @@ the same query is resent with EDNS0 and TCP (if needed). If you
 want the original answer (with TC bit set) and avoid this kind of
 replay, set this flag to false.
 
-=head2 resolver.source
+=head2 resolver.source4
 
-A string that is either an IP address or the exact string C<"os_default">.
-The source address all resolver objects should use when sending queries.
-If C<"os_default">, the OS default address is used.
-Default C<"os_default">.
+A string representation of an IPv4 address or the empty string.
+The source address all resolver objects should use when sending queries over IPv4.
+
+If set to "" (empty string), the OS default IPv4 address is used.
+
+Default: "" (empty string).
+
+=head2 resolver.source6
+
+A string representation of an IPv6 address or the empty string.
+The source address all resolver objects should use when sending queries over IPv6.
+
+If set to "" (empty string), the OS default IPv6 address is used.
+
+Default: "" (empty string).
 
 =head2 net.ipv4
 
@@ -607,12 +723,10 @@ cache.
 
 =head2 asnroots (DEPRECATED)
 
-An arrayref of domain names. Default C<["asnlookup.zonemaster.net",
-"asnlookup.iis.se", "asn.cymru.com"]>.
+An arrayref of domain names. Default C<["asnlookup.zonemaster.net"]>.
 
 The domains will be assumed to be Cymru-style AS lookup zones.
-Normally only the first name in the list will be used, the rest are
-backups in case the earlier ones don't work.
+Only the first name in the list will be used.
 
 =head2 asn_db.style
 
@@ -623,11 +737,28 @@ Default C<"Cymru">.
 =head2 asn_db.sources
 
 An arrayref of domain names when asn_db.style is set to C<"Cymru"> or whois
-servers when asn_db.style is set to C<"RIPE">. Normally only the first item
-in the list will be used, the rest are backups in case the earlier ones don't 
-work.
+servers when asn_db.style is set to C<"RIPE">. Only the first item
+in the list will be used.
 Default C<"asnlookup.zonemaster.net">.
 
+=head2 cache (EXPERIMENTAL)
+
+A hash of hashes. The currently supported keys are C<"redis">.
+
+See more information in L<cache.redis>.
+
+Undefined by default.
+
+=head2 cache.redis (EXPERIMENTAL)
+
+A hashref. The currently supported keys are C<"server"> and C<"expire">.
+
+Specifies the address of the Redis server used to perform global caching
+(C<cache.redis.server>) and an optional expire time (C<cache.redis.expire>).
+
+C<cache.redis.server> must be a string in the form C<host:port>.
+C<cache.redis.expire> must be a non-negative integer and defines a time in seconds. Default 5 seconds.
+
 =head2 logfilter
 
 A complex data structure. Default C<{}>.
@@ -650,10 +781,10 @@ The data under the C<logfilter> key should be structured like this:
              "set"
                 Severity level to set if all conditions match
 
-The hash with conditions should have keys matching the attributes of 
-the log entry that's being filtered (check the translation files to see 
-what they are). The values for the keys should be either a single value 
-that the attribute should be, or an array of values any one of which the 
+The hash with conditions should have keys matching the attributes of
+the log entry that's being filtered (check the translation files to see
+what they are). The values for the keys should be either a single value
+that the attribute should be, or an array of values any one of which the
 attribute should be.
 
 A complete logfilter structure might look like this:
@@ -723,11 +854,16 @@ https://github.com/zonemaster/zonemaster/tree/master/docs/specifications/tests/R
 define the default severity level for some of the messages.
 These specifications are the only authoritative documents on the default
 severity level for the various messages.
-For messages not defined in any of these specifications please refer to the file
-located by L<dist_file("Zonemaster-Engine", "default.profile")|
-File::ShareDir/dist_file>.
-For messages neither defined in test specifications, nor listed in
-C<default.profile>, the default severity level is C<DEBUG>.
+For messages not defined in any of these specifications you can use the
+following command to query the default severity level directly from the actual
+default profile.
+
+```sh
+perl -MZonemaster::Engine::Test -E 'say Zonemaster::Engine::Profile->default->to_json' | jq -S .test_levels
+```
+
+For messages neither defined in test specifications, nor listed in the default
+profile, the default severity level is C<DEBUG>.
 
 I<Note:> Sometimes multiple test cases within the same test module define
 messages for the same tag.
@@ -736,22 +872,87 @@ level for the tag.
 
 =head2 test_cases
 
-An arrayref of names of implemented test cases as listed in the
-L<test case specifications|
-https://github.com/zonemaster/zonemaster/tree/master/docs/specifications/tests/ImplementedTestCases.md>.
+An arrayref of names of implemented test cases (in all lower-case) as listed in the
+L<test case specifications|https://github.com/zonemaster/zonemaster/tree/master/docs/specifications/tests/ImplementedTestCases.md>.
 Default is an arrayref listing all the test cases.
 
-Specifies which test cases to consider when a test module is asked 
-to run of all of its test cases. 
+Specifies which test cases can be run by the testing suite.
 
-Test cases not included here can still be run individually.
+=head2 test_cases_vars.dnssec04.REMAINING_SHORT
 
-The test cases C<basic00>, C<basic01> and C<basic02> are always considered no
-matter if they're excluded from this property.
-This is because part of their function is to verify that the given domain name
-can be tested at all.
+A positive integer value.
+Recommended lower bound for signatures' remaining validity time (in seconds) in
+test case L<DNSSEC04|
+https://github.com/zonemaster/zonemaster/blob/master/docs/specifications/tests/DNSSEC-TP/dnssec04.md>.
+Related to the REMAINING_SHORT message tag from this test case.
+Default C<43200> (12 hours in seconds).
 
-=head1 JSON REPRESENTATION
+=head2 test_cases_vars.dnssec04.REMAINING_LONG
+
+A positive integer value.
+Recommended upper bound for signatures' remaining validity time (in seconds) in
+test case L<DNSSEC04|
+https://github.com/zonemaster/zonemaster/blob/master/docs/specifications/tests/DNSSEC-TP/dnssec04.md>.
+Related to the REMAINING_LONG message tag from this test case.
+Default C<15552000> (180 days in seconds).
+
+=head2 test_cases_vars.dnssec04.DURATION_LONG
+
+A positive integer value.
+Recommended upper bound for signatures' lifetime (in seconds) in the test case
+L<DNSSEC04|
+https://github.com/zonemaster/zonemaster/blob/master/docs/specifications/tests/DNSSEC-TP/dnssec04.md>.
+Related to the DURATION_LONG message tag from this test case.
+Default C<15552000> (180 days in seconds).
+
+=head2 test_cases_vars.zone02.SOA_REFRESH_MINIMUM_VALUE
+
+A positive integer value.
+Recommended lower bound for SOA refresh values (in seconds) in test case
+L<ZONE02|
+https://github.com/zonemaster/zonemaster/blob/master/docs/specifications/tests/Zone-TP/zone02.md>.
+Related to the REFRESH_MINIMUM_VALUE_LOWER message tag from this test case.
+Default C<14400> (4 hours in seconds).
+
+=head2 test_cases_vars.zone04.SOA_RETRY_MINIMUM_VALUE
+
+A positive integer value.
+Recommended lower bound for SOA retry values (in seconds) in test case
+L<ZONE04|
+https://github.com/zonemaster/zonemaster/blob/master/docs/specifications/tests/Zone-TP/zone04.md>.
+Related to the RETRY_MINIMUM_VALUE_LOWER message tag from this test case.
+Default C<3600> (1 hour in seconds).
+
+=head2 test_cases_vars.zone05.SOA_EXPIRE_MINIMUM_VALUE
+
+A positive integer value.
+Recommended lower bound for SOA expire values (in seconds) in test case
+L<ZONE05|
+https://github.com/zonemaster/zonemaster/blob/master/docs/specifications/tests/Zone-TP/zone05.md>.
+Related to the EXPIRE_MINIMUM_VALUE_LOWER message tag from this test case.
+Default C<604800> (1 week in seconds).
+
+=head2 test_cases_vars.zone06.SOA_DEFAULT_TTL_MINIMUM_VALUE
+
+A positive integer value.
+Recommended lower bound for SOA minimum values (in seconds) in test case
+L<ZONE06|
+https://github.com/zonemaster/zonemaster/blob/master/docs/specifications/tests/Zone-TP/zone06.md>.
+Related to the SOA_DEFAULT_TTL_MAXIMUM_VALUE_LOWER message tag from this test case.
+Default C<300> (5 minutes in seconds).
+
+=head2 test_cases_vars.zone06.SOA_DEFAULT_TTL_MAXIMUM_VALUE
+
+A positive integer value.
+Recommended upper bound for SOA minimum values (in seconds) in test case
+L<ZONE06|
+https://github.com/zonemaster/zonemaster/blob/master/docs/specifications/tests/Zone-TP/zone06.md>.
+Related to the SOA_DEFAULT_TTL_MAXIMUM_VALUE_HIGHER message tag from this test case.
+Default C<86400> (1 day in seconds).
+
+=head1 REPRESENTATIONS
+
+=head2 JSON REPRESENTATION
 
 Property names in L</PROFILE PROPERTIES> section correspond to paths in
 a datastructure of nested JSON objects.
@@ -774,8 +975,8 @@ C<net.ipv6> = true has this JSON representation:
         }
     }
 
-=over
+=head2 YAML REPRESENTATION
 
-=back
+Similar to the L</JSON REPRESENTATION> but uses a YAML format.
 
 =cut
